;;; Zastai.el -- Some of my own emacs stuff

;; We need custom (or our own version of it)
;; This bit of code was shamefully stolen from po-mode.el
(eval-and-compile
  (condition-case ()
      (require 'custom)
    (error nil))
  (if (and (featurep 'custom) (fboundp 'custom-declare-variable))
      nil
    (defmacro defgroup (&rest args)
      nil)
    (defmacro defcustom (var value doc &rest args)
      (` (defvar (, var) (, value) (, doc))))))

(defgroup zastai nil
  "A list of variables used by Zastai's functions."
  :tag "Zastai's Customization"
  :group 'local
  )

(defun zastai-unix-to-dos-buffer ()
  "Convert the current buffer to MS-DOG style text. 
This means: append ^M to each line.
Rendered mostly obsolete by Emacs 20's coding system support."
  (interactive)
  (save-excursion
    (message "Dossifying buffer...")
    (goto-char (point-min))
    (while (and (search-forward-regexp "$" nil t) (< (point) (point-max)))
      (replace-match "\015") (forward-char))
    (message "Done!")
  )
)

(defun zastai-dos-to-unix-buffer ()
  "Convert the current buffer to Unix style text. 
This means: strip ^M from each line and ^Z from end of file (if present).
Rendered mostly obsolete by Emacs 20's coding system support."
  (interactive)
  (save-excursion
    (message "Unixifying buffer...")
    (goto-char (point-min))
    (while (and (search-forward-regexp "\015$" nil t) (< (point) (point-max)))
      (replace-match "") (forward-char))
    (goto-char (point-max))
    (backward-char)
    (if (looking-at "\032") (delete-char 1) ;strip ^Z
      )
    (message "Done!")
  )
)

(defcustom zastai-calculator "doscalc"
  "*Program to be run by `zastai-run-calculator'."
  :type 'string
  :tag "Calculator Program"
  :group 'zastai
)

(defun zastai-run-calculator ()
  "Runs the program specified in `zastai-calculator'."
  (interactive)
  (call-process zastai-calculator nil nil nil)
)

; I disabled the use of tabs for indentation (setq indent-tabs-mode nil), so
;  the files I create in Emacs look the same in any other editor. However, this
;  can make existing source files look ugly. Hence this function, which removes
;  any indentation-tabs.
(defun zastai-reindent-c-buffer ()
  "This reindents all lines in the current buffer that were affected by my
change in indent-tabs-mode and tab-width."
  (interactive)
  (save-excursion
    (beginning-of-buffer)
    (while (search-forward "	" nil t)
      (c-indent-command)))
  (message "Done.")
)

;; Key bindings and a menu for my functions
(define-key global-map "\C-cz" (make-sparse-keymap))
(define-key global-map "\C-czc" 'zastai-run-calculator)
(define-key global-map "\C-czx" 'zastai-dos-to-unix-buffer)
(define-key global-map "\C-czd" 'zastai-unix-to-dos-buffer)
(define-key global-map "\C-czm" 'mpuz)
(define-key global-map "\C-czg" 'gomoku)
(define-key global-map "\C-czr" 'zastai-reindent-c-buffer)
(define-key global-map [menu-bar zastai]
  (cons "Zastai" (make-sparse-keymap "Zastai")))
(define-key global-map [menu-bar zastai go]
  '("Gomoku"         . gomoku))
(define-key global-map [menu-bar zastai mpuz]
  '("Mult. Puzzle"   . mpuz))
(define-key global-map [menu-bar zastai calculator]
  '("Run Calculator" . zastai-run-calculator))
(define-key global-map [menu-bar zastai dossify]
  '("Dossify Buffer" . zastai-unix-to-dos-buffer))
(define-key global-map [menu-bar zastai unixify]
  '("Unixify Buffer" . zastai-dos-to-unix-buffer))
(define-key global-map [menu-bar zastai reindent]
  '("Reindent Buffer" . zastai-reindent-c-buffer))

;; Define function keys to actually do something
;; Function Key  1
(define-key global-map [S-f1]      'command-apropos)
(define-key global-map [C-f1]      'info)
;; Function Key  2
(define-key global-map [f2]        'find-tag)
(define-key global-map [S-f2]      'tags-search)
(define-key global-map [C-f2]      'find-tag-regexp)
;; Function Key  3
(define-key global-map [f3]        'isearch-forward)
(define-key global-map [S-f3]      'isearch-backward)
(define-key global-map [C-f3]      'isearch-forward-regexp)
(define-key global-map [C-S-f3]    'isearch-backward-regexp)
;; Function Key  4
(define-key global-map [f4]        'replace-string)
(define-key global-map [C-f4]      'replace-regexp)
;; Function Key  5
(define-key global-map [f5]        'query-replace)
(define-key global-map [S-f5]      'tags-query-replace)
(define-key global-map [C-f5]      'query-replace-regexp)
;; Function Key  6
(define-key global-map [f6]        'other-window)
(define-key global-map [C-f6]      'switch-to-buffer)
;; Function Key  7
(define-key global-map [f7]        'describe-variable)
(define-key global-map [S-f7]      'describe-function)
(define-key global-map [C-f7]      'describe-key)
(define-key global-map [S-C-f7]    'where-is)
(define-key global-map [M-f7]      'describe-face)
(define-key global-map [S-M-f7]    'describe-coding-system)
(define-key global-map [C-M-f7]    'describe-input-method)
(define-key global-map [S-C-M-f7]  'describe-language-environment)
;; Function Key  8
(define-key global-map [f8]        'customize)
(define-key global-map [S-f8]      'customize-group)
(define-key global-map [C-f8]      'customize-variable)
(define-key global-map [S-C-f8]    'customize-face)
(define-key global-map [M-f8]      'customize-apropos)
;; Function Key  9
(define-key global-map [f9]        'delete-window)
(define-key global-map [C-f9]      'delete-other-windows)
(define-key global-map [M-f9]      'delete-frame)
;; Function Key 10
(define-key global-map [f10]       'suspend-emacs)
(define-key global-map [C-f10]     'save-buffers-kill-emacs)
(define-key global-map [C-M-f10]   'kill-emacs)
;; Function Key 11
(define-key global-map [f11]       'font-lock-fontify-buffer)
(define-key global-map [S-f11]     'font-lock-fontify-block)
;; Function Key 12
(define-key global-map [f12]       'kill-buffer)
(define-key global-map [S-f12]     'kill-paragraph)
(define-key global-map [C-f12]     'kill-rectangle)

; Grow/Shrink window
(define-key global-map [C-kp-add]      'enlarge-window)
(define-key global-map [C-kp-subtract] 'shrink-window)

;; Various bindings
(define-key ctl-x-map  "g"             'goto-line)

;; Use keypad enter for newline
;; This is handy in Lisp Interaction buffers, for instance
(define-key function-key-map [kp-enter] [10])

(provide 'zastai)
